<?php

/**
 * Drupal _comment() hook.
 */
function comment_comment(&$comment, $op) {
  switch ($op) {
    case 'update':
      spam_content_update($comment, 'comment');
      break;
    case 'insert':
      spam_content_insert($comment, 'comment');
      break;
    case 'delete':
      spam_content_delete($comment, 'comment');
      break;
    case 'form':
      if (is_array($comment['cid'])) {
        _spam_comment_cid($comment['cid']['#value']);
      }
      break;
  }
}

/**
 * Cache the comment id to be sure it's available when we need it.
 */
function _spam_comment_cid($id = NULL) {
  static $cid = 0;

  if (isset($id) && is_numeric($id)) {
    $cid = $id;
  }

  return $cid;
}

/** 
 * Spam module _spamapi() hook.
 */
function comment_spamapi($op, $arg1 = NULL, $arg2 = NULL, $arg3 = NULL) {
  switch ($op) {
    case 'content_module':
      // Register with the spam api as a content type module.
      return 'comment';

    case 'content_id':
      // Tell the spam module the id of a given comment.
      if (is_object($arg1)) {
        // The delete hook uses an object instead of an array.
        $arg1 = (array)$arg1;
      }
      return _spam_comment_cid($arg1['cid']);

    case 'content_types':
      // Register the "comment" content type with the spam module.
      return array(array(
        'name' => t('comments'),
        'module' => t('comment'),
        'title' => t('Comments'),
        'description' => t('Check this box to filter comments for spam.'),
        'default_value' => 1,
      ));

    case 'filter_content_type':
      return(variable_get('spam_filter_comments', 1));

    case 'filter_fields':
      // Tell spam module which fields it should scan for spam.
      $fields['main'] = array('subject', 'comment');
      if (is_object($arg1)) {
        // The delete hook uses an object instead of an array.
        $arg1 = (array)$arg1;
      }
      if (isset($arg1['author'])) {
        $fields['other'][] = 'author';
      }
      if (isset($arg1['name'])) {
        $fields['other'][] = 'name';
      }
      if (isset($arg1['mail'])) {
        $fields['other'][] = 'mail';
      }
      if (isset($arg1['homepage'])) {
        $fields['other'][] = 'homepage';
      }
      return $fields;

    case 'error_form':
      $form = array();
      $form['comment'] = array(
        '#type' => 'fieldset',
        '#title' => 'Comment',
      );
      $form['comment']['title'] = array(
        '#type' => 'textfield',
        '#title' => t('Subject'),
        '#value' => $arg1['subject'],
        '#disabled' => TRUE,
      );
      $form['comment']['body'] = array(
        '#type' => 'textarea',
        '#title' => t('Comment'),
        '#value' => $arg1['comment'],
        '#disabled' => TRUE,
      );
      return $form;

    case 'load':
      return db_fetch_object(db_query('SELECT * FROM {comments} WHERE cid = %d', $arg1));

    case 'title':
      return db_result(db_query('SELECT subject FROM {comments} WHERE cid = %d', $arg1));

    case 'edit_link':
      return "comment/edit/$arg1";

    case 'status':
      $status = db_result(db_query('SELECT status FROM {comments} WHERE cid = %d', $arg1));
      if ($status == COMMENT_PUBLISHED) {
        return SPAM_PUBLISHED;
      }
      else {
        return SPAM_NOT_PUBLISHED;
      }

    case 'process_form':
      // Hook to scan comment before it is inserted into the database.
      if ($arg1 == 'comment_form' && is_array($arg2)) {
        $comment = $arg2['#post'];
        if (is_array($comment) && $comment['op'] == t('Post comment')) {
          spam_scan($comment, 'comment');
        }
      }
      break;

    case 'hostname':
      return db_result(db_query('SELECT hostname FROM {comments} WHERE cid = %d', $arg1));

    case 'link':
      if (is_object($arg1) && isset($arg1->cid)) {
        return spam_links('comment', $arg1->cid, $arg1);
      }
      break;

    case 'redirect':
      $nid = db_result(db_query('SELECT nid FROM {comments} WHERE cid = %d', $arg1));
      return drupal_goto("/node/$nid", NULL, "comment-$arg1");

    case 'overview_filter_join':
      return 'INNER JOIN {comments} c ON t.content_id = c.cid';

    case 'overview_filter_where':
      switch ($arg1) {
        case 'title':
          return "c.subject LIKE '%%%s%%'";
        case 'status':
          return 'c.status = %d';
      }

    case 'publish':
    case 'unpublish':
      // Only update comments that exist and need to be updated.
      $nid = db_result(db_query('SELECT nid FROM {comments} WHERE cid = %d AND status = %d', $arg1, $op == 'publish' ? COMMENT_NOT_PUBLISHED : COMMENT_PUBLISHED));
      if ($nid) {
        $action = comment_operations($op);
        db_query($action[$op][1], $arg1);
        _comment_update_node_statistics($nid);
      }
      break;
  }
}
